/* ========================================================================= *
 *                                                                           *
 *                 The Apache Software License,  Version 1.1                 *
 *                                                                           *
 *             Copyright (c) 2002 The Apache Software Foundation.            *
 *                           All rights reserved.                            *
 *                                                                           *
 * ========================================================================= *
 *                                                                           *
 * Redistribution and use in source and binary forms,  with or without modi- *
 * fication, are permitted provided that the following conditions are met:   *
 *                                                                           *
 * 1. Redistributions of source code  must retain the above copyright notice *
 *    notice, this list of conditions and the following disclaimer.          *
 *                                                                           *
 * 2. Redistributions  in binary  form  must  reproduce the  above copyright *
 *    notice,  this list of conditions  and the following  disclaimer in the *
 *    documentation and/or other materials provided with the distribution.   *
 *                                                                           *
 * 3. The end-user documentation  included with the redistribution,  if any, *
 *    must include the following acknowlegement:                             *
 *                                                                           *
 *       "This product includes  software developed  by the Apache  Software *
 *        Foundation <http://www.apache.org/>."                              *
 *                                                                           *
 *    Alternately, this acknowlegement may appear in the software itself, if *
 *    and wherever such third-party acknowlegements normally appear.         *
 *                                                                           *
 * 4. The names  "The Jakarta  Project",  and  "Apache  Software Foundation" *
 *    must not  be used  to endorse  or promote  products derived  from this *
 *    software without  prior written  permission.  For written  permission, *
 *    please contact <apache@apache.org>.                                    *
 *                                                                           *
 * 5. Products derived from this software may not be called "Apache" nor may *
 *    "Apache" appear in their names without prior written permission of the *
 *    Apache Software Foundation.                                            *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES *
 * INCLUDING, BUT NOT LIMITED TO,  THE IMPLIED WARRANTIES OF MERCHANTABILITY *
 * AND FITNESS FOR  A PARTICULAR PURPOSE  ARE DISCLAIMED.  IN NO EVENT SHALL *
 * THE APACHE  SOFTWARE  FOUNDATION OR  ITS CONTRIBUTORS  BE LIABLE  FOR ANY *
 * DIRECT,  INDIRECT,   INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL *
 * DAMAGES (INCLUDING,  BUT NOT LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS *
 * OR SERVICES;  LOSS OF USE,  DATA,  OR PROFITS;  OR BUSINESS INTERRUPTION) *
 * HOWEVER CAUSED AND  ON ANY  THEORY  OF  LIABILITY,  WHETHER IN  CONTRACT, *
 * STRICT LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN *
 * ANY  WAY  OUT OF  THE  USE OF  THIS  SOFTWARE,  EVEN  IF  ADVISED  OF THE *
 * POSSIBILITY OF SUCH DAMAGE.                                               *
 *                                                                           *
 * ========================================================================= *
 *                                                                           *
 * This software  consists of voluntary  contributions made  by many indivi- *
 * duals on behalf of the  Apache Software Foundation.  For more information *
 * on the Apache Software Foundation, please see <http://www.apache.org/>.   *
 *                                                                           *
 * ========================================================================= */

package org.apache.commons.launcher;

import java.io.File;
import java.io.PrintStream;

/**
 * A class for detecting if the parent JVM that launched this process has
 * terminated.
 *
 * @author Patrick Luby
 */
public class ParentListener extends Thread {

    //------------------------------------------------------------------ Fields

    /**
     * Cached heartbeat file.
     */
    private File heartbeatFile = null;

    //------------------------------------------------------------ Constructors

    /**
     * Validates and caches a lock file created by the parent JVM.
     *
     * @param heartbeatFile the lock file that the parent JVM will delete when
     *  it wants this process to exit
     */
    public ParentListener(File heartbeatFile) {

        this.heartbeatFile = heartbeatFile;

    }

    //----------------------------------------------------------------- Methods

    /**
     * Periodically check that the parent JVM has not terminated by checking
     * if {@link System#out} and {@link System#err} are open. If
     * either stream is in an error state, the parent JVM is assumed to be
     * terminated and this method will invoke {@link System#exit(int)}.
     * This method <b>must</b> be executed <b>before</b> either
     * {@link System#setOut(PrintStream)} or {@link System#setErr(PrintStream)}
     * is ever invoked.
     * <p>
     * In addition, we check if the heartbeat file has been deleted. If so,
     * we assume the parent is requesting that this process terminate itself.
     */ 
    public void run() {

        // Save System.out and System.err in case the application invokes
        // System.setOut() or System.setErr() later
        PrintStream out = System.out;
        PrintStream err = System.err;
        while (heartbeatFile.exists() && !out.checkError() && !err.checkError()) {
            // Wait a while before the next loop
            yield();
            try {
                sleep(3000);
            } catch (Exception e) {}
        }

        // Clean up before exiting
        heartbeatFile.delete();

        // Exit this process since the parent JVM has exited
        System.exit(0);

    }

}
